@@ -1,4 +1,4 @@
-#' Read files created of the statistical spreadsheet 'jamovi' (www.jamovi.org)
+#' Read files created of the statistical spreadsheet 'jamovi' (<https://www.jamovi.org>)
 #'
 #' @param fleInp Name (including the path, if required) of the 'jamovi'-file to be read ("FILENAME.omv"; default: "")
 #' @param useFlt Apply filters (remove the lines where the filter is set to 0; default: FALSE)?
@@ -56,90 +56,79 @@ read_omv <- function(fleInp = "", useFlt = FALSE, rmMsVl = FALSE, sveAtt = TRUE,
     if (strBin) strHdl <- getHdl(fleInp, "strings.bin", "rb");
 
     # process meta-data
-    if (! all(grepl(grpMta, names(mtaDta)))) stop("Unimplemeted field in the meta data");
+    if (!all(grepl(grpMta, names(mtaDta)))) stop("Unimplemeted field in the meta data");
 
-    # rowCount, columnCount
+    # determine rows and columns and create data frame
     rowNum <- mtaDta$rowCount
     colNum <- mtaDta$columnCount
     if (length(mtaDta$fields) != colNum) stop("Number of fields in the metadata is not matching up the number of columns.");
+    dtaFrm <- stats::setNames(data.frame(matrix(NA, nrow = rowNum, ncol = colNum)), sapply(mtaDta$fields, "[[", "name"))
+    fltLst <- c()
 
     # iterate through fields
-    lblLst <- c()
-    fltLst <- c()
     for (i in seq_len(colNum)) {
         # type: determines the format in the binary file
         if        (chkFld(mtaDta$fields[[i]], "type", "integer")) {
-            colRaw <- as.data.frame(readBin(binHdl,   integer(), n = rowNum));
+            crrCol <- readBin(binHdl,   integer(), n = rowNum);
         } else if (chkFld(mtaDta$fields[[i]], "type", "number"))  {
-            colRaw <- as.data.frame(readBin(binHdl,    double(), n = rowNum));
+            crrCol <- readBin(binHdl,    double(), n = rowNum);
         } else if (chkFld(mtaDta$fields[[i]], "type", "string"))  {
-            colRaw <- as.data.frame(readBin(strHdl, character(), n = rowNum));
-                                    readBin(binHdl,   integer(), n = rowNum);
+            crrCol <- readBin(strHdl, character(), n = rowNum);
+                      readBin(binHdl,   integer(), n = rowNum);
         } else {
             stop(sprintf("Variable type \"%s\" not implemented.", mtaDta$fields[[i]]$type));
         }
 
-        # name, description
-        nmeCrr <- mtaDta$fields[[i]]$name;
-        lblCrr <- mtaDta$fields[[i]]$description;
-
-        lblLst <- c(lblLst, lblCrr)
+        # name
+        crrNme <- mtaDta$fields[[i]]$name;
 
         # value labels
-        if (any(nmeCrr == names(xtdDta))) {
+        if (any(crrNme == names(xtdDta))) {
             if        (chkFld(mtaDta$fields[[i]], "columnType", "Filter") || chkFld(mtaDta$fields[[i]], "name", "^Filter [0-9]+$")) {
-                colRaw[[1]] <- as.logical(colRaw[[1]]);
+                crrCol <- as.logical(crrCol);
                 fltLst <- c(fltLst, i);
             } else if (chkFld(mtaDta$fields[[i]], "columnType", "Data|Recoded")) {
-                colRaw[[1]] <- factor(colRaw[[1]], levels = unlist(sapply(xtdDta[[nmeCrr]]$labels, function(m) m[1])),
-                                                   labels = unlist(sapply(xtdDta[[nmeCrr]]$labels, function(m) m[2])),
-                                                   ordered = chkFld(mtaDta$fields[[i]], "measureType", "Ordinal"));
-                if    (chkFld(mtaDta$fields[[i]], "dataType",   "Integer")) {
-                    attr(colRaw[[1]], "values") <- unlist(sapply(xtdDta[[nmeCrr]]$labels, function(m) as.integer(m[1])));
+                crrCol <- factor(crrCol, levels = unlist(sapply(xtdDta[[crrNme]]$labels, function(m) m[1])),
+                                         labels = unlist(sapply(xtdDta[[crrNme]]$labels, function(m) m[2])),
+                                         ordered = chkFld(mtaDta$fields[[i]], "measureType", "Ordinal"));
+                if (chkFld(mtaDta$fields[[i]], "dataType", "Integer")) {
+                    if (identical(sort(levels(crrCol)), c("0", "1")))        crrCol <- as.logical(gsub("^1$", "TRUE", gsub("^0$", "FALSE", crrCol)))
+                    if (identical(sort(levels(crrCol)), c("FALSE", "TRUE"))) crrCol <- as.logical(crrCol)
                 }
             } else {
                 stop(sprintf("Error when reading value label - likely the column type is not implemented (yet): %s - %s - %s",
-                             nmeCrr, mtaDta$fields[[i]]$dataType, mtaDta$fields[[i]]$columnType));
+                             crrNme, mtaDta$fields[[i]]$dataType, mtaDta$fields[[i]]$columnType));
             }
         }
 
-        if (i == 1) {
-            names(colRaw) <- nmeCrr;
-            dtaFrm <- colRaw;
-        } else {
-            dtaFrm[nmeCrr] <- colRaw;
-        }
+        dtaFrm[[crrNme]] <- crrCol;
 
-        if (lblCrr != "")
-            attr(dtaFrm[[nmeCrr]], "jmv-desc") <- lblCrr;
+        if (chkFld(mtaDta$fields[[i]], "description", ".+")) attr(dtaFrm[[crrNme]], "jmv-desc") <- mtaDta$fields[[i]]$description;
+        if (chkFld(mtaDta$fields[[i]], "measureType", "ID")) attr(dtaFrm[[crrNme]], "jmv-id")   <- TRUE;
 
-        if (chkFld(mtaDta$fields[[i]], "measureType", "ID")) {
-            attr(dtaFrm[[nmeCrr]], "jmv-id") <- TRUE;
-        }
-
-        dtaFrm[[nmeCrr]] <- setAtt("missingValues",   mtaDta$fields[[i]], dtaFrm[[nmeCrr]])
+        dtaFrm[[crrNme]] <- setAtt("missingValues",   mtaDta$fields[[i]], dtaFrm[[crrNme]])
 
         if (sveAtt) {
-            dtaFrm[[nmeCrr]] <- setAtt(names(mtaFld), mtaDta$fields[[i]], dtaFrm[[nmeCrr]])
+            dtaFrm[[crrNme]] <- setAtt(names(mtaFld), mtaDta$fields[[i]], dtaFrm[[crrNme]])
         }
 
         if (rmMsVl) {
-            mssLst <- attr(dtaFrm[[nmeCrr]], "missingValues");
+            mssLst <- attr(dtaFrm[[crrNme]], "missingValues");
             if (length(mssLst) > 0) {
-               attCrr <- attributes(dtaFrm[[nmeCrr]]);
-               rmvLvl <- rep(FALSE, length(levels(dtaFrm[[nmeCrr]])));
+               crrAtt <- attributes(dtaFrm[[crrNme]]);
+               rmvLvl <- rep(FALSE, length(levels(dtaFrm[[crrNme]])));
                for (j in seq_along(mssLst)) {
-                   dtaFrm[[nmeCrr]][eval(parse(text = paste0("dtaFrm[[\"", nmeCrr, "\"]]", mssLst[[j]])))] <- NA;
-                   rmvLvl <- rmvLvl | eval(parse(text = paste0("levels(dtaFrm[[\"", nmeCrr, "\"]]) ", mssLst[j])));
+                   dtaFrm[[crrNme]][eval(parse(text = paste0("dtaFrm[[\"", crrNme, "\"]]", mssLst[[j]])))] <- NA;
+                   rmvLvl <- rmvLvl | eval(parse(text = paste0("levels(dtaFrm[[\"", crrNme, "\"]]) ", mssLst[j])));
                }
-               dtaFrm[[nmeCrr]] <- dtaFrm[[nmeCrr]][, drop = TRUE];
-               attCrr$missingValues <- list();
-               attCrr$values <- attCrr$values[!rmvLvl];
-               dtaFrm[[nmeCrr]] <- setAtt(setdiff(names(attCrr), names(attributes(dtaFrm[[nmeCrr]]))), attCrr, dtaFrm[[nmeCrr]]);
-               rm(attCrr, rmvLvl);
+               dtaFrm[[crrNme]] <- dtaFrm[[crrNme]][, drop = TRUE];
+               crrAtt$missingValues <- list();
+               crrAtt$values <- crrAtt$values[!rmvLvl];
+               dtaFrm[[crrNme]] <- setAtt(setdiff(names(crrAtt), names(attributes(dtaFrm[[crrNme]]))), crrAtt, dtaFrm[[crrNme]]);
+               rm(crrAtt, rmvLvl);
             }
         }
-        rm(colRaw);
+        rm(crrCol);
     }
 
     # close and remove the binary file(s)
@@ -214,50 +203,109 @@ read_omv <- function(fleInp = "", useFlt = FALSE, rmMsVl = FALSE, sveAtt = TRUE,
     dtaFrm
 }
 
-fndSyn <- function(resElm = NULL) {
-    if (chkSyn(resElm)) {
-        resElm[["preformatted"]]
-    } else if (utils::hasName(resElm, "group") && length(resElm[["group"]]) > 0) {
-        for (obj in resElm[["group"]][["elements"]]) {
-            ret <- Recall(obj);
-            if (!is.null(ret)) return(ret);
-        }
+
+# =================================================================================================
+# read_all: for reading data files from various formats (incl. functions that are called)
+
+read_all <- function(fleInp = "", usePkg = c("foreign", "haven"), selSet = "", ...) {
+    if (nchar(fleInp) == 0) stop("File name to the input data file needs to be given as parameter (fleInp = ...).");
+
+    # check whether the file exists
+    fleInp <- fmtFlI(fleInp, maxLng = 1)
+    varArg <- list(...);
+    usePkg <- match.arg(usePkg);
+    dtaFrm <- NULL;
+
+    # OMV
+    if        (hasExt(fleInp, c("omv"))) {
+        dtaFrm <- tryCatch(do.call(read_omv, adjArg("read_omv", list(fleInp = fleInp), varArg, "fleInp")),
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # CSV
+    } else if (hasExt(fleInp, c("csv"))) {
+        dtaFrm <- tryCatch(rmvQtn(do.call(utils::read.table, adjArg("read.table", list(file = fleInp, sep = ",",  quote = "\"", header = TRUE, fill = TRUE), varArg, "file"))),
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # TSV
+    } else if (hasExt(fleInp, c("tsv"))) {
+        dtaFrm <- tryCatch(rmvQtn(do.call(utils::read.table, adjArg("read.table", list(file = fleInp, sep = "\t", quote = "\"", header = TRUE, fill = TRUE), varArg, "file"))),
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # Rdata
+    } else if (hasExt(fleInp, c("rdata", "rda"))) {
+        dtaFrm <- tryCatch({
+                             load(fleInp, rdaTmp <- new.env());
+                             if (length(rdaTmp) != 1 && selSet == "") stop("The Rdata-file must include only one object.");
+                             rdaTmp[[ifelse(selSet == "", names(rdaTmp)[1], selSet)]]
+                           },
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # RDS
+    } else if (hasExt(fleInp, c("rds"))) {
+        dtaFrm <- tryCatch(do.call(readRDS, adjArg("readRDS", list(file = fleInp), varArg, "file")),
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # SPSS (haven / foreign)
+    } else if (hasExt(fleInp, c("sav", "zsav"))) {
+        dtaFrm <- tryCatch({
+                             if        (usePkg == "haven"   && hasPkg("haven"))   {
+                                 hvnTmp <- haven::as_factor(do.call(haven::read_sav, adjArg("haven::read_sav", list(file = fleInp), varArg, "file")), only_labelled = TRUE);
+                                 hvnAdj(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.spss", "display_width"), jmvLbl = TRUE)
+                             } else if (usePkg == "foreign" && hasPkg("foreign")) {
+                                 fgnLbl(do.call(foreign::read.spss, adjArg("foreign::read.spss", list(file = fleInp, to.data.frame = TRUE), varArg, c("file", "to.data.frame"))))
+                             } else {
+                                 stop(sprintf("In order to read the SPSS-file \"%s\" either of the R-packages \"haven\" or \"foreign\" needs to be installed.", basename(fleInp)));
+                             }
+                           },
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # Stata (haven / foreign)
+    } else if (hasExt(fleInp, c("dta"))) {
+        dtaFrm <- tryCatch({
+                             # more recent versions of the Stata-format require "haven" and can't be read with foreign
+                             usePkg <- ifelse(grepl("^<stata_dta><header>", readBin(fleInp, character(), n = 1)), "haven", usePkg)
+                             if        (usePkg == "haven"   && hasPkg("haven"))   {
+                                 hvnTmp <- haven::as_factor(do.call(haven::read_dta, adjArg("haven::read_dta", list(file = fleInp), varArg, "file")), only_labelled = TRUE);
+                                 hvnAdj(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.stata", "display_width"), jmvLbl = TRUE)
+                             } else if (usePkg == "foreign" && hasPkg("foreign")) {
+                                 fgnLbl(do.call(foreign::read.dta, adjArg("foreign::read.dta", list(file = fleInp), varArg, c("file"))))
+                             } else {
+                                 stop(sprintf("In order to read the Stata-file \"%s\" either of the R-packages \"haven\" or \"foreign\" needs to be installed.", basename(fleInp)));
+                             }
+                           },
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # SAS data (haven)
+    } else if (hasExt(fleInp, c("sas7bdat", "sd2", "sd7"))) {
+        dtaFrm <- tryCatch({
+                             if        (usePkg == "haven"   && hasPkg("haven"))   {
+                                 hvnTmp <- haven::as_factor(do.call(haven::read_sas, adjArg("haven::read_sas", list(data_file = fleInp), varArg, "data_file")), only_labelled = TRUE);
+                                 hvnAdj(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.sas", "display_width"), jmvLbl = TRUE)
+                             } else {
+                                 stop(sprintf("In order to read the SAS-file \"%s\" the R-packages \"haven\" needs to be installed.", basename(fleInp)));
+                             }
+                           },
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
+    # SAS-transport-files (haven / foreign)
+    } else if (hasExt(fleInp, c("xpt", "stx", "stc"))) {
+        dtaFrm <- tryCatch({
+                             if        (usePkg == "haven"   && hasPkg("haven"))   {
+                                 hvnTmp <- haven::as_factor(do.call(haven::read_xpt, adjArg("haven::read_xpt", list(file = fleInp), varArg, "file")), only_labelled = TRUE);
+                                 hvnAdj(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.stata", "display_width"), jmvLbl = TRUE)
+                             } else if (usePkg == "foreign" && hasPkg("foreign")) {
+                                 fgnLbl(do.call(foreign::read.xport, adjArg("foreign::read.xport", list(file = fleInp), varArg, c("file"))))
+                             } else {
+                                 stop(sprintf("In order to read the SAS-transport-file \"%s\" either of the R-packages \"haven\" or \"foreign\" needs to be installed.", basename(fleInp)));
+                             }
+                           },
+                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
     }
+
+    # check whether the input data are a data frame with the correct dimensions
+    chkDtF(dtaFrm);
+
+    # check whether all attributes conform with unicode and do some cleaning if required
+    dtaFrm <- rplAtt(dtaFrm);
+
+    dtaFrm
 }
 
-chkSyn <- function(resElm = NULL) {
-   # checks whether the results element is a syntax entry
-   # it has to have the name syntax and the preformatted attribute must not be empty
-   utils::hasName(resElm, "name")         && resElm[["name"]]         == "syntax" &&
-   utils::hasName(resElm, "preformatted") && resElm[["preformatted"]] != ""
-}
 
-getTxt <- function(fleOMV = "", crrFle = "") {
-    crrTxt <- readLines(crrHdl <- getHdl(fleOMV, crrFle), warn = FALSE);
-    clsHdl(crrHdl); rm(crrHdl);
-
-    # depending on whether the original was a JSON file or not, return the appropriate result
-    if (hasExt(crrFle, "json")) {
-        crrTxt <- rjson::fromJSON(crrTxt, simplify = FALSE);
-    }
-    crrTxt
-}
-
-getHdl <- function(fleOMV = "", crrFle = "", crrMde = "r") {
-    zip::unzip(fleOMV, crrFle, exdir = tempdir(), junkpaths = TRUE);
-    crrFle <- file.path(tempdir(), list.files(path = tempdir(), pattern = basename(crrFle)));
-    if (length(crrFle) == 0) {
-        stop(sprintf("The file \"%s\" could not be extracted from \"%s\". Please register an issue.", crrFle, fleOMV));
-    }
-    file(crrFle, crrMde)
-}
-
-clsHdl <- function(crrHdl = NULL) {
-    crrFle <- summary(crrHdl)$description;
-    close(crrHdl);
-    unlink(crrFle);
-    rm(crrFle, crrHdl);
-}
+# =================================================================================================
+# helper functions for read_omv and read_all (in alphabetical order)
 
 chkMnf <- function(fleOMV = "", fleMnf = c("")) {
     if (length(fleMnf) < 1) {
@@ -282,134 +330,87 @@ chkMnf <- function(fleOMV = "", fleMnf = c("")) {
                                    basename(fleOMV), lstMnf[[i]][1], crrVer, lstMnf[[i]][-1]));
         }
     }
+
+    TRUE
 }
 
-
-# =================================================================================================
-# read_all: for reading data files from various formats (incl. functions that are called)
-
-read_all <- function(fleInp = "", usePkg = c("foreign", "haven"), selSet = "", ...) {
-    # check whether the file exists
-    chkFle(fleInp)
-    varArg <- list(...);
-    usePkg <- match.arg(usePkg);
-    dtaFrm <- NULL;
-
-    # OMV
-    if        (hasExt(fleInp, c("omv"))) {
-        dtaFrm <- tryCatch(do.call(read_omv, adjArg("read_omv", list(fleInp = fleInp), varArg, "fleInp")),
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    # CSV
-    } else if (hasExt(fleInp, c("csv"))) {
-        dtaFrm <- tryCatch(rmvQtn(do.call(utils::read.table, adjArg("read.table", list(file = fleInp, sep = ",",  header = TRUE, fill = TRUE), varArg, "file"))),
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    # TSV
-    } else if (hasExt(fleInp, c("tsv"))) {
-        dtaFrm <- tryCatch(rmvQtn(do.call(utils::read.table, adjArg("read.table", list(file = fleInp, sep = "\t", header = TRUE, fill = TRUE), varArg, "file"))),
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    # Rdata
-    } else if (hasExt(fleInp, c("rdata", "rda"))) {
-        dtaFrm <- tryCatch({
-                             load(fleInp, rdaTmp <- new.env());
-                             if (length(rdaTmp) != 1 && selSet == "") stop("The Rdata-file must include only one object.");
-                             rdaTmp[[ifelse(selSet == "", names(rdaTmp)[1], selSet)]]
-                           },
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    # RDS
-    } else if (hasExt(fleInp, c("rds"))) {
-        dtaFrm <- tryCatch(do.call(readRDS, adjArg("readRDS", list(file = fleInp), varArg, "file")),
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    # SPSS (haven / foreign)
-    } else if (hasExt(fleInp, c("sav", "zsav"))) {
-        dtaFrm <- tryCatch({
-                             if        (usePkg == "haven"   && hasPkg("haven"))   {
-                                 hvnTmp <- haven::as_factor(do.call(haven::read_sav, adjArg("haven::read_sav", list(file = fleInp), varArg, "file")), only_labelled = TRUE);
-                                 hvnDrp(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.spss", "display_width"))
-                             } else if (usePkg == "foreign" && hasPkg("foreign")) {
-                                 fgnLbl(do.call(foreign::read.spss, adjArg("foreign::read.spss", list(file = fleInp, to.data.frame = TRUE), varArg, c("file", "to.data.frame"))))
-                             } else {
-                                 stop(sprintf("In order to read the SPSS-file \"%s\" either of the R-packages \"haven\" or \"foreign\" needs to be installed.", basename(fleInp)));
-                             }
-                           },
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    # Stata (haven / foreign)
-    } else if (hasExt(fleInp, c("dta"))) {
-        dtaFrm <- tryCatch({
-                             if        (usePkg == "haven"   && hasPkg("haven"))   {
-                                 hvnTmp <- haven::as_factor(do.call(haven::read_dta, adjArg("haven::read_dta", list(file = fleInp), varArg, "file")), only_labelled = TRUE);
-                                 hvnDrp(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.stata", "display_width"))
-                             } else if (usePkg == "foreign" && hasPkg("foreign")) {
-                                 fgnLbl(do.call(foreign::read.dta, adjArg("foreign::read.dta", list(file = fleInp), varArg, c("file"))))
-                             } else {
-                                 stop(sprintf("In order to read the Stata-file \"%s\" either of the R-packages \"haven\" or \"foreign\" needs to be installed.", basename(fleInp)));
-                             }
-                           },
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    # SAS data (haven)
-    } else if (hasExt(fleInp, c("sas7bdat", "sd2", "sd7"))) {
-        dtaFrm <- tryCatch({
-                             if        (usePkg == "haven"   && hasPkg("haven"))   {
-                                 hvnTmp <- haven::as_factor(do.call(haven::read_sas, adjArg("haven::read_sas", list(data_file = fleInp), varArg, "data_file")), only_labelled = TRUE);
-                                 hvnDrp(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.sas", "display_width"))
-                             } else {
-                                 stop(sprintf("In order to read the SAS-file \"%s\" the R-packages \"haven\" needs to be installed.", basename(fleInp)));
-                             }
-                           },
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    } else if (hasExt(fleInp, c("xpt", "stx", "stc"))) {
-        dtaFrm <- tryCatch({
-                             if        (usePkg == "haven"   && hasPkg("haven"))   {
-                                 hvnTmp <- haven::as_factor(do.call(haven::read_xpt, adjArg("haven::read_xpt", list(file = fleInp), varArg, "file")), only_labelled = TRUE);
-                                 hvnDrp(as.data.frame(hvnTmp@.Data, col.names = names(hvnTmp)), c("format.stata", "display_width"))
-                             } else if (usePkg == "foreign" && hasPkg("foreign")) {
-                                 fgnLbl(do.call(foreign::read.xport, adjArg("foreign::read.xport", list(file = fleInp), varArg, c("file"))))
-                             } else {
-                                 stop(sprintf("In order to read the SAS-transport-file \"%s\" either of the R-packages \"haven\" or \"foreign\" needs to be installed.", basename(fleInp)));
-                             }
-                           },
-                           error = function(errMsg) tryErr(fleInp, errMsg), warning = function(wrnMsg) tryWrn(fleInp, wrnMsg));
-    }
-
-    # check whether the input data are a data frame with the correct dimensions
-    chkDtF(dtaFrm);
-
-    # check whether all attributes conform with unicode and do some cleaning if required
-    dtaFrm <- rplAtt(dtaFrm);
-
-    dtaFrm
+chkSyn <- function(resElm = NULL) {
+   # checks whether the results element is a syntax entry
+   # it has to have the name syntax and the preformatted attribute must not be empty
+   utils::hasName(resElm, "name")         && resElm[["name"]]         == "syntax" &&
+   utils::hasName(resElm, "preformatted") && resElm[["preformatted"]] != ""
 }
 
-tryErr <- function(fleInp = "", errMsg = NULL) {
-    message(sprintf("File \"%s\" couldn\'t be read.\nThe error message was: %s\n", basename(fleInp), conditionMessage(errMsg)));
-    return(NULL)
-}
-
-tryWrn <- function(fleInp = "", wrnMsg = NULL) {
-    message(sprintf("Warnings were issued when reading the file \"%s\".\nThe warning was: %s\n", basename(fleInp), conditionMessage(wrnMsg)));
-    return(NULL)
+clsHdl <- function(crrHdl = NULL) {
+    crrFle <- summary(crrHdl)$description;
+    close(crrHdl);
+    unlink(crrFle);
+    rm(crrFle, crrHdl);
 }
 
 fgnLbl <- function(dtaFrm = NULL) {
-    if (! is.null(attr(dtaFrm, "variable.labels"))) {
+    if (!is.null(attr(dtaFrm, "variable.labels"))) {
         varLbl <- trimws(attr(dtaFrm, "variable.labels"));
         for (crrCol in names(dtaFrm)) {
-            if (varLbl[[crrCol]] != "") {
-                attr(dtaFrm[[crrCol]], "label") <- varLbl[[crrCol]];
+            if (crrCol %in% names(varLbl) && varLbl[[crrCol]] != "") {
+                attr(dtaFrm[[crrCol]], "jmv-desc") <- varLbl[[crrCol]];
             }
         }
         attr(dtaFrm, "variable.labels") <- NULL;
     }
+
     dtaFrm
 }
 
-hvnDrp <- function(dtaFrm = NULL, rmvAtt = c()) {
+fndSyn <- function(resElm = NULL) {
+    if (chkSyn(resElm)) {
+        resElm[["preformatted"]]
+    } else if (utils::hasName(resElm, "group") && length(resElm[["group"]]) > 0) {
+        for (obj in resElm[["group"]][["elements"]]) {
+            ret <- Recall(obj);
+            if (!is.null(ret)) return(ret);
+        }
+    }
+}
+
+getHdl <- function(fleOMV = "", crrFle = "", crrMde = "r") {
+    tryCatch(expr  = {
+                 zip::unzip(fleOMV, crrFle, exdir = tempdir(), junkpaths = TRUE);
+                 file(file.path(tempdir(), list.files(path = tempdir(), pattern = basename(crrFle))), crrMde)
+             },
+             error = function(errMsg) {
+                 message(sprintf("The file \"%s\" could not be extracted from \"%s\".\nPlease send the file to sebastian.jentschke@uib.no!\nError message: %s\n", crrFle, fleOMV, errMsg));
+                 NULL
+             }
+        )
+}
+
+
+getTxt <- function(fleOMV = "", crrFle = "") {
+    crrTxt <- readLines(crrHdl <- getHdl(fleOMV, crrFle), warn = FALSE);
+    clsHdl(crrHdl); rm(crrHdl);
+
+    # depending on whether the original was a JSON file or not, return the appropriate result
+    if (hasExt(crrFle, "json")) {
+        crrTxt <- rjson::fromJSON(crrTxt, simplify = FALSE);
+    }
+
+    crrTxt
+}
+
+hvnAdj <- function(dtaFrm = NULL, rmvAtt = c(), jmvLbl = FALSE) {
    for (crrCol in names(dtaFrm)) {
        for (crrAtt in rmvAtt) {
              if (! is.null(attr(dtaFrm[[crrCol]], crrAtt))) {
                  attr(dtaFrm[[crrCol]], crrAtt) <- NULL
              }
         }
+        if (jmvLbl && !is.null(attr(dtaFrm[[crrCol]], "label"))) {
+            attr(dtaFrm[[crrCol]], "jmv-desc") <- attr(dtaFrm[[crrCol]], "label")
+            attr(dtaFrm[[crrCol]], "label")    <- NULL
+        }
     }
+
     dtaFrm
 }
 
@@ -417,6 +418,7 @@ rmvQtn <- function(dtaFrm = NULL) {
     for (crrCol in names(which(sapply(dtaFrm, is.character)))) {
         dtaFrm[[crrCol]] <- trimws(gsub("\"", "", dtaFrm[[crrCol]]))
     }
+
     dtaFrm
 }
 
@@ -447,13 +449,27 @@ rplAtt <- function(dtaFrm = NULL) {
 }
 
 rplStr <- function(strMod = "", crrAtt = "") {
-    # lstRpl is defined in globals.R
-    for (i in seq_len(dim(lstRpl)[2])) {
-        strMod <- gsub(lstRpl[1, i], lstRpl[2, i], strMod);
-    }
-    if (! all(validEnc(strMod))) {
-        stop(sprintf("The current data set still contains an invalid character in attribute: \"%s\".", crrAtt));
+    # encode as UTF-8 (to enforce valid encoding)
+    strMod <- enc2utf8(strMod)
+    if (any(grepl("<[0-9,a-f][0-9,a-f]>", strMod))) {
+        # lstRpl is defined in globals.R
+        for (i in seq_len(dim(lstRpl)[2])) {
+            strMod <- gsub(lstRpl[1, i], lstRpl[2, i], strMod);
+        }
+        if (any(grepl("<[0-9,a-f][0-9,a-f]>", strMod))) {
+            stop(sprintf("The current data set still contains an invalid character (\"%s\") in attribute: \"%s\".", strMod, crrAtt));
+        }
     }
 
     strMod
 }
+
+tryErr <- function(fleInp = "", errMsg = NULL) {
+    message(sprintf("File \"%s\" couldn\'t be read.\nThe error message was: %s\n", basename(fleInp), conditionMessage(errMsg)));
+    return(NULL)
+}
+
+tryWrn <- function(fleInp = "", wrnMsg = NULL) {
+    message(sprintf("Warnings were issued when reading the file \"%s\".\nThe warning was: %s\n", basename(fleInp), conditionMessage(wrnMsg)));
+    return(NULL)
+}
